@@ -6,7 +6,6 @@ A Python wrapper for the SVO Filter Profile Service
 from glob import glob
 import inspect
 import os
-import pickle
 from pkg_resources import resource_filename
 import warnings
 import itertools
@@ -14,10 +13,9 @@ import itertools
 import astropy.table as at
 import astropy.io.votable as vo
 import astropy.units as q
-import astropy.constants as ac
 from astropy.utils.exceptions import AstropyWarning
+from astroquery.svo_fps import SvoFps
 from bokeh.plotting import figure, show
-from bokeh.models import Label
 import bokeh.palettes as bpal
 import numpy as np
 
@@ -132,37 +130,49 @@ class Filter:
 
         else:
 
-            # Get list of filters
-            files = glob(filter_directory+'*')
-            no_ext = {f.replace('.txt', ''): f for f in files}
-            bands = [os.path.basename(b) for b in no_ext]
-            fp = os.path.join(filter_directory, band)
-            filepath = no_ext.get(fp, fp)
+            # List of all bands on file
+            bands = filters(filter_directory)
 
-            # If the filter is missing, ask what to do
-            if band not in bands:
+            # Read file if the band is in the filter directory
+            if band in bands:
 
-                err = """No filters match {}\n\nCurrent filters: {}\n\nA full list of available filters from the\nSVO Filter Profile Service can be found at\nhttp: //svo2.cab.inta-csic.es/theory/fps3/\n\nPlace the desired filter XML file in your\nfilter directory and try again.""".format(filepath, ', '.join(bands))
+                # Get the file
+                filepath = glob(os.path.join(filter_directory, band + '*'))[0]
 
-                raise IOError(err)
+                # Get the first line to determine format
+                with open(filepath) as f:
+                    top = f.readline()
 
-            # Get the first line to determine format
-            with open(filepath) as f:
-                top = f.readline()
+                # Read in XML file
+                if top.startswith('<?xml'):
 
-            # Read in XML file
-            if top.startswith('<?xml'):
+                    self.load_xml(filepath)
 
-                self.load_xml(filepath)
+                # Read in txt file
+                elif filepath.endswith('.txt'):
 
-            # Read in txt file
-            elif filepath.endswith('.txt'):
+                    self.load_txt(filepath)
 
-                self.load_txt(filepath)
+                else:
 
+                    raise TypeError("File must be XML or ascii format.")
+
+            # Otherwise try a Web query or throw an error
             else:
+              
+                err = """No filters match {}\n\nFILTERS ON FILE: {}\n\nA full list of available filters from the\nSVO Filter Profile Service can be found at\nhttp: //svo2.cab.inta-csic.es/theory/fps3/\n\nTry again with the desired filter as '<facility>/<instrument>.<filter_name>', e.g. '2MASS/2MASS.J'""".format(band, ', '.join(bands))
 
-                raise TypeError("File must be XML or ascii format.")
+                # Try a web query
+                if '/' in band:
+                    try:
+                        self.load_web(band)
+                    except:
+                        raise IndexError(err)
+                
+                else:
+                    
+                    # Or throw an error
+                    raise IndexError(err)
 
         # Set the wavelength and throughput
         self._wave_units = q.AA
@@ -490,14 +500,28 @@ class Filter:
         file: str
             The filepath
         """
+        # Get the transmission data from the file
         self.raw = np.genfromtxt(filepath, unpack=True)
 
         # Convert to Angstroms if microns
         if self.raw[0][-1] < 100:
             self.raw[0] = self.raw[0] * 10000
 
+        # Load it into the object
+        self.filterID = os.path.splitext(os.path.basename(filepath))[0]
+        self.load_raw()
+
+    def load_raw(self, wave_units='AA'):
+        """
+        The raw data to calculate the filter properties from
+
+        Parameters
+        ----------
+        data: sequence
+            The wavelength and throughput for the filter
+        """
         funit = q.erg / q.s / q.cm**2 / q.AA
-        self.WavelengthUnit = str(q.AA)
+        self.WavelengthUnit = wave_units
         self.ZeroPointUnit = str(funit)
         x, f = self.raw
 
@@ -509,7 +533,6 @@ class Filter:
 
         # Calculate the filter's properties
         self.ZeroPoint = np.trapz(f * x * vega, x=x) / np.trapz(f * x, x=x)
-        self.filterID = os.path.splitext(os.path.basename(filepath))[0]
         self.WavelengthPeak = x[np.argmax(f)]
         self.WavelengthMin = x[np.where(f > f.max() / 100.)][0]
         self.WavelengthMax = x[np.where(f > f.max() / 100.)][-1]
@@ -531,6 +554,24 @@ class Filter:
         self.pixels_per_bin = self.raw.shape[-1]
         self.n_bins = 1
 
+    def load_web(self, filt):
+        """Load the filter from a Web query
+
+        Parameters
+        ----------
+        filt: str
+            The '<facility>/<instrument>.<filter_name>' of the filter, e.g. '2MASS/2MASS.J'
+        """
+        # Get the transmission data for the filter
+        data = SvoFps.get_transmission_data(filt)
+
+        # Make into arrays
+        self.raw = np.array([np.array(data['Wavelength']), np.array(data['Transmission'])])
+
+        # Load it into the object
+        self.filterID = filt
+        self.load_raw(wave_units=str(data['Wavelength'].unit))
+
     def load_xml(self, filepath):
         """Load the filter from a txt file
 
@@ -806,7 +847,7 @@ def color_gen(colormap='viridis', key=None, n=15):
     yield from itertools.cycle(palette)
 
 
-def filters(filter_directory=None, update=False, fmt='table', **kwargs):
+def filters(filter_directory=None):
     """
     Get a list of the available filters
 
@@ -827,73 +868,13 @@ def filters(filter_directory=None, update=False, fmt='table', **kwargs):
     if filter_directory is None:
         filter_directory = resource_filename('svo_filters', 'data/filters/')
 
-    # Get the pickle path and make sure file exists
-    p_path = os.path.join(filter_directory, 'filter_list.p')
-    updated = False
-    if not os.path.isfile(p_path):
-        os.system('touch {}'.format(p_path))
+    # Get list of files from dir
+    files = glob(os.path.join(filter_directory, '*'))
 
-    if update:
+    # Grab the basename as the filter name
+    bands = [b.replace(filter_directory, '').replace('.txt', '').replace('.xml', '') for b in files]
 
-        print('Loading filters into table...')
-
-        # Get all the filters (except the pickle)
-        files = glob(filter_directory+'*')
-        files = [f for f in files if not f.endswith('.p')]
-        bands = [os.path.basename(b) for b in files]
-        tables = []
-
-        for band in bands:
-
-            # Load the filter
-            band = band.replace('.txt', '')
-            filt = Filter(band, **kwargs)
-            filt.Band = band
-
-            # Put metadata into table with correct dtypes
-            info = filt.info(True)
-            vals = [float(i) if i.replace('.', '').replace('-', '')
-                    .replace('+', '').isnumeric() else i
-                    for i in info['Values']]
-            dtypes = np.array([type(i) for i in vals])
-            table = at.Table(np.array([vals]), names=info['Attributes'],
-                             dtype=dtypes)
-
-            tables.append(table)
-
-            del filt, info, table
-
-        # Write to the pickle
-        with open(p_path, 'wb') as file:
-            pickle.dump(at.vstack(tables), file)
-
-    # Load the saved pickle
-    data = {}
-    if os.path.isfile(p_path):
-        with open(p_path, 'rb') as file:
-            data = pickle.load(file)
-
-    # Return the data
-    if data:
-
-        if fmt == 'dict':
-            data = {r[0]: {k: r[k].value if hasattr(r[k], 'unit') else r[k]
-                    for k in data.keys()[1:]} for r in data}
-
-        else:
-
-            # Add Band as index
-            data.add_index('Band')
-
-        return data
-
-    # Or try to generate it once
-    else:
-        if not updated:
-            updated = True
-            filters(update=True)
-        else:
-            print('No filters found in', filter_directory)
+    return bands
 
 
 def rebin_spec(spec, wavnew, oversamp=100, plot=False):
